#!/usr/bin/env python

class Config(object):
    """ """

    def __init__(self, row):
        """ """
        
        self.name = row[0]
        self.command = row[1] 

        if len(row) > 2:
            self.formatter = row[2]
        else:
            self.formatter = "extract"
        
        if len(row) > 3:
            self.position = int(row[3])
        else:
            self.position = 1



#!/usr/bin/env python

from versions.config import Config
from versions.program import Program
from subprocess import Popen, DEVNULL, PIPE

class Versions(object):
    """Wraps the current environment"""
    
    def __init__(self, outputter):
        """Initializes the environment wrapper

        Args:
            outputter (Outputter): The outputter of the program details.
        """

        self.outputter = outputter
        self.items = []

    def add_version(self, config):
        """Runs the program to retrieve the program version

        Args:
            args (list): The program name and required args to return the version info.
            formatter (Formatter): The outputter of the program details.
        """

        try:
            p = Popen(config.command.split(), stdin=PIPE, stdout=PIPE, stderr=PIPE)
            output, err = p.communicate(b"input data that is passed to subprocess' stdin")
            rc = p.returncode

            if rc is 0:
            
                output = output.rstrip().decode("utf-8") 
                output = getattr(self, config.formatter)(config, output)
                # output = locals()[config.formatter](config, output)
                # output = self.extract(config, output)
                self.items.append(output)

            else:
                print(err)
        
        except FileNotFoundError as error:
            program = Program(config.name, "Not installed or on PATH")
            self.items.append(program)

    def extract(self, config, output):
        """ """

        items = output.split()
        return Program(config.name, items[config.position])

    def extract_go(self, config, output):
        """ """

        items = output.split()
        return Program(config.name, items[2][2:])

    def extract_docker(self, config, output):
        """ """

        items = output.split()
        return Program(config.name, items[2][:-1])

    def output(self):
        """Outputs to based on the provides args"""

        self.outputter.output(self.items)


#!/usr/bin/env python

import os
import csv
import argparse
from versions.config import Config
from versions.versions import Versions
from versions.outputter import JsonOutputter

root_parser = argparse.ArgumentParser(description='The command line tools for https://www.delineate.io',add_help=True)
sub_parsers = root_parser.add_subparsers(dest='command')
versions = sub_parsers.add_parser('versions', help='blame people')
versions.add_argument('--out', type=str, help='Output location for the Json file')

def versions_list():
    pass

def main(command_line=None):
    """Runs the program"""

    args = root_parser.parse_args(command_line)

    if args.command == 'versions':
        
        versions = Versions(JsonOutputter(args.out))

        path = os.path.abspath(os.path.dirname(__file__))
        path = os.path.join(path, "versions/programs.csv")

        with open(path) as csv_file:
            csv_reader = csv.reader(csv_file, delimiter = ',')
            for row in csv_reader:
                versions.add_version(Config(row))
        
        versions.output()

if __name__ == '__main__':   # pragma: no cover
    main()


#!/usr/bin/env python

class Program(object):
    """ """
    
    def __init__(self, name, version):
        """ """

        self.name = str(name)

        if version[:1] is not "v":
            self.version = "v{}".format(version)
        else:
            self.version = version


import setuptools

VERSION = '0.0.12'

with open("pip.md", "r") as file:
    long_description = file.read()

setuptools.setup(
    name="delineate-io",
    version=VERSION,
    py_modules=['app'],
    author="https://www.delineate.io",
    author_email="oss@delineate.io",
    description="Utility for the current environment versions",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url="https://github.com/delineateio/delineateio.tools",
    download_url='https://github.com/delineateio/delineateio.tools/tarball/{}'.format(VERSION),
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': ['delineate-io=app:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


#!/usr/bin/env python

import json

class JsonOutputter(object):
    """Outputs the program info as Json to either the console or file"""

    def __init__(self, file_name = None):
        """Initializes the Json Outputter

        Args:
            file_name (str): Optional name of the file.
        """

        self.file_name = file_name

    def output(self, items):
        """Outputs the program info as Json

        Args:
            items (list): The list of program info.
        """

        jdict = {item.name: item.version for item in items}

        if self.file_name is None:
            self.output_to_console(jdict)
        else:
            self.output_to_file(jdict)

    def output_to_console(self, jdict):
        """Outputs the program info as Json to the console

        Args:
            jdict (dict): The dict of program info.
        """

        print(json.dumps(jdict, sort_keys=True, indent=4))

    def output_to_file(self, jdict):
        """Outputs the program info as Json to a file

        Args:
            jdict (dict): The dict of program info.
        """
        
        with open(self.file_name, 'w') as file:
            json.dump(jdict, file, sort_keys=True, indent=4)

    
